#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# dataport-schuposh-saml-permissions
# listener module
#
# Copyright 2020-2022 Univention GmbH
#
# http://www.univention.de/
#
# All rights reserved.
#
# The source code of this program is made available
# under the terms of the GNU Affero General Public License version 3
# (GNU AGPL V3) as published by the Free Software Foundation.
#
# Binary versions of this program provided by Univention to you as
# well as other copyrighted, protected or trademarked materials like
# Logos, graphics, fonts, specific documentations and configurations,
# cryptographic keys etc. are subject to a license agreement between
# you and Univention and not subject to the GNU AGPL V3.
#
# In the case you use this program under the terms of the GNU AGPL V3,
# the program is provided in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License with the Debian GNU/Linux or Univention distribution in file
# /usr/share/common-licenses/AGPL-3; if not, see
# <http://www.gnu.org/licenses/>.

__package__ = '' 	# workaround for PEP 366

import copy
import listener
import univention.debug
import univention.admin.uldap
from univention.config_registry import ConfigRegistry


name = 'dataport-schuposh-itsl-saml-permissions'
description = 'Manage itslearning SAML SP permissions for the group itslearner-<schoolname>'
filter = '(&(name=itslearner-*)(objectClass=posixGroup))'
attributes = []
modrdn = "1"

ucr = ConfigRegistry()
ucr.load()
ldap_base = ucr.get('ldap/base')
domain_name = ucr.get('domainname')
server_role = ucr.get('server/role')

service_provider = 'https://eu1.itslearning.com/integrations/samlmetadata/saml2v2/2'

saml_sp_DN = 'SAMLServiceProviderIdentifier={},cn=saml-serviceprovider,cn=univention,{}'.format(service_provider, ldap_base)


def handler(dn, new, old, command=''):
	univention.debug.debug(univention.debug.LISTENER, univention.debug.WARN, "{}: DN {}".format(name, dn))
	if not server_role == 'domaincontroller_master':
		# univention.debug.debug(univention.debug.LISTENER, univention.debug.WARN, "{}: Not on a DC master, not doing anything".format(name))
		return
	if not new:
		univention.debug.debug(univention.debug.LISTENER, univention.debug.WARN, "{}: Group is not being added, not doing anything".format(name))
		return

	listener.setuid(0)
	lo, po = univention.admin.uldap.getAdminConnection()
	listener.unsetuid()

	modlist = []

	existing_objectClasses = lo.getAttr(dn, 'objectClass')
	if 'univentionSAMLEnabledGroup' not in existing_objectClasses:
		new_objectClasses = copy.deepcopy(existing_objectClasses)
		new_objectClasses.append('univentionSAMLEnabledGroup')
		modlist.append(('objectClass', existing_objectClasses, new_objectClasses))

	existing_saml_permissions = set(lo.getAttr(dn, 'enabledServiceProviderIdentifierGroup'))
	new_saml_permissions = copy.deepcopy(existing_saml_permissions)
	new_saml_permissions.update(saml_sp_DN)
	if new_saml_permissions.symmetric_difference(existing_saml_permissions):
		modlist.append(('enabledServiceProviderIdentifierGroup', list(existing_saml_permissions), list(new_saml_permissions)))

	if modlist:
		univention.debug.debug(univention.debug.LISTENER, univention.debug.WARN, "{}: LO modify with group {} and modlist {}".format(name, dn, modlist))
		lo.modify(dn, modlist)

	univention.debug.debug(univention.debug.LISTENER, univention.debug.WARN, "{}: Done for DN {}".format(name, dn))


def clean():
	return


def postrun():
	return
